﻿package com.castledragmire.GPSMapping;

//GUI
import android.os.Bundle;
import android.app.Activity;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.ImageView;
import android.view.View;
import android.view.View.OnClickListener;
import android.content.res.Resources;
import android.widget.Toast;
import android.text.TextWatcher;
import android.text.Editable;
//Settings
import android.content.SharedPreferences;
import java.util.regex.Pattern;

public class Settings extends Activity
{
	//The settings and their properties
	public class GPSMappingSetting
	{
		//Constant Properties
		public final String Name, SettingExplanation;
		public final Pattern ConfirmRegExp;
		public final TextView Label;
		public final EditText TextBox;
		public final ImageView HelpButton;

		//Realtime properties
		public String Value;
		public boolean IsValid=false;

		GPSMappingSetting(SharedPreferences P, Resources R, String PackageName, String Name, String ConfirmRegExp, String Default)
		{
			//Passed variables
			this.Name=Name;
			this.ConfirmRegExp=Pattern.compile(ConfirmRegExp);

			//Resource loaded variables
			SettingExplanation=getString(R.getIdentifier("Settings_Explanation_"+Name, "string", PackageName));
			Label=(TextView)findViewById(R.getIdentifier("Settings_Label_"+Name, "id", PackageName));
			TextBox=(EditText)findViewById(R.getIdentifier(Name, "id", PackageName));
			HelpButton=(ImageView)findViewById(R.getIdentifier("Help_"+Name, "id", PackageName));

			//Hook resources
			TextBox.addTextChangedListener(new ConfirmChange_Listener());
			HelpButton.setOnClickListener(DisplayHelp);

			//Load the setting
			TextBox.setText(P.getString(Name, Default));
		}

		//Confirm a setting is valid
		public void ConfirmChange(CharSequence TheText)
		{
			Value=TheText.toString();
			IsValid=ConfirmRegExp.matcher(TheText).matches();
			Label.setTextColor(IsValid ? 0xFFFFFFFF : 0xFFFF0000); //Set the background color of the label to red if invalid
		}
		private class ConfirmChange_Listener implements TextWatcher {
			@Override public void afterTextChanged(Editable s) { ConfirmChange(s); }
			@Override public void beforeTextChanged(CharSequence s, int start, int count, int after) {}
			@Override public void onTextChanged(CharSequence s, int start, int before, int count) {}
		};
	}
	private GPSMappingSetting MySettings[];

	//Initiate the settings dialog
	@Override public void onCreate(Bundle savedInstanceState)
	{
		//Create the layout
		super.onCreate(savedInstanceState);
		setContentView(R.layout.settings);

		//Load the settings
		SharedPreferences P=getSharedPreferences("MyPrefs", 0);
		Resources Res=getResources();
		MySettings=new GPSMappingSetting[5];
		String PN=getPackageName();
		MySettings[0]=new GPSMappingSetting(P,Res,PN,"UserName",	"^[-A-Za-z0-9_ \\.]{1,25}$","");
		MySettings[1]=new GPSMappingSetting(P,Res,PN,"Password",	"^.{1,4096}$",				"");
		MySettings[2]=new GPSMappingSetting(P,Res,PN,"MinTime",		"^[1-9][0-9]?$",			"5");
		MySettings[3]=new GPSMappingSetting(P,Res,PN,"MinDistance",	"^[0-9]{1,2}$",				"1");
		MySettings[4]=new GPSMappingSetting(P,Res,PN,"SendDataURL",	"^https?://.+$",			getString(R.string.Settings_Default_SendDataURL));
	}

	//Save the settings and update the service
	@Override protected void onStop()
	{
		//Store the settings
		SharedPreferences Pref=getSharedPreferences("MyPrefs", 0);
		SharedPreferences.Editor edit=Pref.edit();
		String ErrorMessage=null;
		for(GPSMappingSetting Setting : MySettings)
		{
			edit.putString(Setting.Name, Setting.Value);
			if(!Setting.IsValid && ErrorMessage==null)
				ErrorMessage=Setting.SettingExplanation;
		}
		boolean AllSettingsValid=(ErrorMessage==null);
		edit.putBoolean("AllSettingsValid", AllSettingsValid); //Disable the service if all settings are not valid
		edit.commit();

		//Update the service
		if(!AllSettingsValid || !Pref.getBoolean("IsEnabled", true)) //Stop the service and display the error
		{
			GPSMappingService.StopService(this);
			Toast.makeText(getApplicationContext(), "GPS Mapping is stopped due to invalid setting: "+ErrorMessage, Toast.LENGTH_LONG).show();
		}
		else //Start/Update the service
			GPSMappingService.UpdateVariables(true, this);

		super.onStop();
	}

	//Display an appropriate help message
	private OnClickListener DisplayHelp=new OnClickListener() {  @Override public void onClick(View v)
	{
		GPSMappingSetting Setting=null;
		for(int i=0;i<MySettings.length;i++)
			if(MySettings[i].HelpButton==v)
			{
				Setting=MySettings[i];
				break;
			}
		Toast.makeText(getApplicationContext(), Setting.SettingExplanation, Toast.LENGTH_LONG).show();
	} };
}